@@ -3,27 +3,27 @@ use strict;
 use warnings;
 use Expect qw( exp_continue );
 use POSIX qw(:sys_wait_h :unistd_h); # For WNOHANG
-use Hash::Merge::Simple qw (merge);
 use Module::Load;
 use App::Tel::HostRange qw (check_hostrange);
 use App::Tel::Passwd;
 use App::Tel::Color;
 use App::Tel::Macro;
+use App::Tel::Merge qw ( merge );
+use App::Tel::Expect;
 use Time::HiRes qw ( sleep );
 use v5.10;
 
-
 =head1 NAME
 
 App::Tel - A script for logging into devices
 
 =head1 VERSION
 
-0.201509
+0.201601
 
 =cut
 
-our $VERSION = '0.201509';
+our $VERSION = '0.201601';
 
 
 =head1 SYNOPSIS
@@ -48,7 +48,7 @@ under the same terms as Perl itself.
 
 #### GLOBALS
 # For reasons related to state I needed to make $_winch_it global
-# because it needs to be written to inside signals.
+# because it needs to be written inside signals.
 my $_winch_it=0;
 
 sub _winch_handler {
@@ -56,7 +56,7 @@ sub _winch_handler {
 }
 
 sub _winch {
-    my $session = shift->{'session'};
+    my $session = shift->session;
     # these need to be wrapped in eval or you get Given filehandle is not a
     # tty in clone_winsize_from if you call winch() under a scripted
     # environment like rancid (or just under par, or anywhere there is no pty)
@@ -79,20 +79,46 @@ Creates a new App::Tel object.
 =cut
 
 sub new {
-    my $infile = IO::File->new;
-    $infile->IO::File::fdopen( \*STDIN, 'r' );
+    my ($class, %args) = @_;
 
     my $self = {
-        'stdin'         => Expect->exp_init($infile),
-        'stdin_fileno'  => $infile->fileno,
+        'stdin'         => Expect->exp_init(\*STDIN),
         'connected'     => 0,
         'enabled'       => 0,
         'title_stack'   => 0,
         'log_stdout'    => 1,
         'profile'       => {},
+        'perl'          => $args{perl} || '',
+        'opts'          => $args{opts},
+        'colors'        => App::Tel::Color->new($args{opts}->{d}),
     };
 
-    bless($self, 'App::Tel');
+    return bless($self, $class);
+}
+
+
+=head2 go
+
+    $self->go('hostname');
+
+Handles all the routines of connecting, enable, looping and disconnection.
+This is the method called by bin/tel for each host.
+
+=cut
+
+sub go {
+    my ($self, $host) = @_;
+
+    # default profile always loads before anything else.  replace == 1
+    $self->profile('default', 1);
+    $self->profile($self->{opts}->{P}) if ($self->{opts}->{P});
+    $self->hostname($host);
+    $self->login($self->hostname);
+    $self->profile($self->{opts}->{A}) if ($self->{opts}->{A});
+    if ($self->connected) {
+        $self->enable->logging->control_loop->disconnect;
+    }
+
     return $self;
 }
 
@@ -109,27 +135,28 @@ If you supply a true value, it will hard_close the session.
 =cut
 
 sub disconnect {
-    my $self = shift;
-    my $hard = shift;
+    my ($self, $hard) = @_;
     $self->{profile} = {};
     $self->{timeout} = $self->{opts}->{t} ? $self->{opts}->{t} : 90;
     $self->{banners} = undef;
     $self->{methods} = ();
     $self->connected(0);
-    $self->{colors}=();
+    $self->{colors}=App::Tel::Color->new($self->{opts}->{d});
     $self->{enabled}=0;
+
     if ($self->{title_stack} > 0) {
         $self->{title_stack}--;
         print "\e[23t";
     }
+
     if ($hard) {
         $self->session->hard_close();
     } else {
         $self->session->soft_close();
     }
+    return $self;
 }
 
-
 =head2 send
 
     $self->send("text\r");
@@ -139,25 +166,19 @@ Wrapper for Expect's send() method.
 =cut
 
 sub send {
-    return shift->{'session'}->send(@_);
+    return shift->session->send(@_);
 }
 
 =head2 expect
 
     $self->expect("text");
 
-Wrapper for Expect's expect() method.  If you don't specify a timeout this
-will use the default script timeout.
+Wrapper for Expect's expect() method.
 
 =cut
 
 sub expect {
-    my $self = shift;
-    if ($#_ < 2) {
-        return $self->{'session'}->expect($self->{'timeout'}, @_);
-    } else {
-        return $self->{'session'}->expect(@_);
-    }
+    return shift->session->expect(@_);
 }
 
 =head2 load_config
@@ -172,8 +193,9 @@ by calling load_config with an argument:
 
 sub load_config {
     my $self = shift;
+    my $xdg = $ENV{XDG_CONFIG_HOME} || "$ENV{HOME}/.config/";
     my @configs = @_;
-    @configs = ( "/etc/telrc", "/usr/local/etc/telrc", "$ENV{HOME}/.telrc2") if (!@configs);
+    @configs = ( "/etc/telrc", "/usr/local/etc/telrc", "$ENV{HOME}/.telrc2", "$xdg/telrc") if (!@configs);
     push(@configs, $ENV{TELRC}) if (defined($ENV{TELRC}));
     our $telrc;
     my $config;
@@ -191,10 +213,9 @@ sub load_config {
     }
 
     # load global syntax highlighting things if found
-    for(@{$config->{syntax}}) {
-        App::Tel::Color::load_syntax($_,$self->{opts}->{d});
-    }
-    return $config;
+    $self->{colors}->load_syntax($config->{syntax});
+    $self->{config} = $config;
+    return $self;
 }
 
 =head2 hostname
@@ -212,8 +233,7 @@ translated because of an alias.  The final hostname is stored and returned.
 =cut
 
 sub hostname {
-    my $self = shift;
-    my $hostname = shift;
+    my ($self, $hostname) = @_;
 
     if (!defined($hostname)) {
         return $self->{hostname};
@@ -293,6 +313,10 @@ sub methods {
     return $self->{methods};
 }
 
+# this is overridable in the individual test file to connect a special
+# way if the method is listed as "test"
+sub _test_connect { undef }
+
 sub _banners {
     my $self = shift;
     return $self->{banners} if ($self->{banners});
@@ -313,11 +337,17 @@ sub _banners {
 Find the router by hostname/regex and load the config associated with it.
 Load a profile for it if there is one.
 
+If none of the router lines match then this does not merge another profile,
+but will return the existing profile.  That may not be what you want in all
+cases.  The workaround for it would be to clear profiles before calling this
+with $self->profile('default',1);
+
+I'm trying to decide if there should be a better way to do this.
+
 =cut
 
 sub rtr_find {
-    my $self = shift;
-    my $host = shift;
+    my ($self, $host) = @_;
     my $profile = $self->{'profile'};
     my $config = $self->{'config'};
 
@@ -332,31 +362,27 @@ sub rtr_find {
 
     # if the host specified a profile to load then load it here
     if (defined($profile->{profile})) {
-        return $self->profile($profile->{profile});
+        $self->profile($profile->{profile});
     }
 
-    return $profile;
+    return $self;
 }
 
 =head2 profile
 
    $profile = $self->profile;
-   $profile = $self->profile('profilename', $replace);
+   $profile = $self->profile('a_example_profile', $replace);
 
 Called without arguments this will return the current profile.  If called with
-a profilename it will load that new profile on top of whatever profile
+a profile name it will load that new profile on top of whatever profile
 currently exists.  You can set the second argument to true if you want to
 replace the current profile with the new one.
 
 =cut
 
 sub profile {
-    my $self = shift;
-    my $profile_arg = shift;
-    my $replace = shift;
+    my ($self, $profile_arg, $replace) = @_;
     my $stdin = $self->{'stdin'};
-    my $config = $self->{'config'};
-    my $session = $self->{'session'};
     my $profile = $self->{'profile'};
 
     return $profile if (!defined($profile_arg));
@@ -367,7 +393,7 @@ sub profile {
     }
 
     foreach(split(/\+/, $profile_arg)) {
-        $profile = merge($profile, $config->{profile}{$_});
+        $profile = merge($profile, $self->{'config'}->{profile}{$_});
 
         # load handlers for profile
         if ($profile->{handlers}) {
@@ -376,14 +402,13 @@ sub profile {
             }
         }
         # load syntax highlight
-        if ($profile->{syntax}) {
-            App::Tel::Color::load_syntax($profile->{syntax},$self->{opts}->{d});
-        }
+        $self->{colors}->load_syntax($profile->{syntax});
         $profile->{profile_name}=$_;
     }
 
     # add some sane defaults if the profile doesn't have them
-    $profile->{'user'} ||= $ENV{'USER'};
+    $profile->{'user'} ||= $ENV{'USER'} || $ENV{'LOGNAME'};
+    # need to warn if user is still not defined?
     $self->{'profile'}=$profile;
     return $profile;
 }
@@ -392,7 +417,7 @@ sub _stty_rows {
     my $new_rows = shift;
     eval {
         Module::Load::load Term::ReadKey;
-        my ($columns, $rows, $xpix, $ypix) = GetTerminalSize(\*STDOUT);
+        my ($columns, undef, $xpix, $ypix) = GetTerminalSize(\*STDOUT);
         SetTerminalSize($columns, $new_rows, $xpix, $ypix, \*STDOUT);
     };
 
@@ -413,8 +438,7 @@ you can call it with $self->password('enable');
 =cut
 
 sub password {
-    my $self = shift;
-    my $type = shift;
+    my ($self, $type) = @_;
     my $profile = $self->profile;
     my $router = $self->{hostname};
 
@@ -463,14 +487,13 @@ session.
 =cut
 
 sub session {
-    my $self = shift;
-    my $renew = shift;
+    return $_[0]->{'session'} if (!$_[1] && defined($_[0]->{'session'}));
 
-    return $self->{'session'} if (!$renew && defined($self->{'session'}));
+    my $self = shift;
     my $session = $self->{'session'};
 
     $session->soft_close() if ($session && $session->pid());
-    $session = new Expect;
+    $session = Expect->new;
 
     # install sig handler for window size change
     $SIG{WINCH} = \&_winch_handler;
@@ -490,8 +513,7 @@ This sets up the session.  If there already is a session open it closes and open
 =cut
 
 sub connect {
-    my $self = shift;
-    my @arguments = shift;
+    my ($self, @arguments) = @_;
 
     $self->connected(0);
     my $session = $self->session(1);
@@ -515,9 +537,8 @@ is at a prompt of some kind.
 =cut
 
 sub connected {
-    my $self = shift;
-    my $status = shift;
-    if ($status) {
+    my ($self, $status) = @_;
+    if (defined($status)) {
         $self->{connected}=$status;
     }
 
@@ -526,9 +547,10 @@ sub connected {
 
 =head2 enable
 
-    my $enabled = $self->enable;
+    $self->enable;
 
-if enablecmd is set then this method attempts to enable the user
+if enablecmd is set then this method attempts to enable the user.  Returns
+$self.
 
 =cut
 
@@ -551,36 +573,42 @@ sub enable {
     }
 
     $self->{enabled}=1;
-    return $self->{enabled};
+    return $self;
 }
 
+=head2 enabled
+
+    my $enabled = $self->enabled;
+
+Check if enabled.  This returns true if the $self->enable() method succeeds.
+
+=cut
+
+sub enabled { $_[0]->{enabled} };
+
 =head2 login
 
-    my $something = $self->login("hostname");
+    $self->login("hostname");
 
 Cycles through the connection methods trying each in the order specified by
-the profile until we successfully connect to the host.  Returns connected
-status (true or false).
+the profile until we successfully connect to the host.  Returns $self.
 
 =cut
 
 sub login {
-    my $self = shift;
-    my $hostname = shift;
+    my ($self, $hostname) = @_;
 
     # dumb stuff to alias $rtr to the contents of $self->{'profile'}
-    # needed because we reload the profile in the expect loop and can't update
-    # the alias.
+    # needed because we can reload the profile inside the expect loop
+    # and can't update the alias.
     our $rtr;
     *rtr = \$self->{'profile'};
 
     my $ssho = '-o StrictHostKeyChecking=no';
     if (defined($rtr->{sshoptions}) && scalar $rtr->{sshoptions} > 0) {
-        my @sshoptions = @{$rtr->{sshoptions}};
-        $ssho = '-o '. join(' -o ', @sshoptions);
+        $ssho = '-o '. join(' -o ', @{$rtr->{sshoptions}});
     }
-
-    my $cipher = $rtr->{ciphertype} ? ('-c ' . $rtr->{ciphertype}) : '';
+    $ssho .= $rtr->{ciphertype} ? " -c $rtr->{ciphertype}" : '';
 
     # because we use last METHOD; in anonymous subs this suppresses the
     # warning of "exiting subroutine via last;"
@@ -595,49 +623,42 @@ sub login {
     $rtr->{username_prompt} ||= qr/[Uu]ser[Nn]ame:|[Ll]ogin:/;
     $rtr->{password_prompt} ||= qr/[Pp]ass[Ww]ord/;
 
-    $self->{port} ||= $self->{opts}->{p}; # get port from CLI
-    $self->{port} ||= $rtr->{port};       # or the profile
+    $self->{port} ||= $self->{opts}->{p} || $rtr->{port}; # get port from CLI or the profile
     # if it's not set in the profile or CLI above, it gets set in the
     # method below, but needs to be reset on each loop to change from
     # telnet to ssh defaults
 
+    my $family = '';
+    $family = '-4' if ($self->{opts}->{4});
+    $family = '-6' if ($self->{opts}->{6});
+
     METHOD: for (@{$self->methods}) {
-        my $allied_shit=0;
-
         my $p = $self->{port};
-
-        if ($_ eq 'ssh')        { $p ||= 22; $self->connect("ssh -p $p -l $rtr->{user} $ssho $cipher $hostname"); }
-        elsif ($_ eq 'telnet')  { $p ||= 23; $self->connect("telnet $hostname $p"); }
-        # for testing. can pass an expect script to the other side and use it's output as our input.
-        elsif ($_ eq 'exec')    { $self->connect($hostname); }
+        if    ($_ eq 'ssh')     { $p ||= 22; $self->connect("ssh $family -p $p -l $rtr->{user} $ssho $hostname"); }
+        elsif ($_ eq 'telnet')  { $p ||= ''; $self->connect("telnet $family $hostname $p"); }
+        elsif ($_ eq 'test')    { $self->_test_connect($hostname); }
         else { die "No program defined for method $_\n"; }
 
         # suppress stdout if needed
-        $self->{'session'}->log_stdout($self->{log_stdout});
+        $self->session->log_stdout($self->{log_stdout});
 
         # need to make this optional
         # also need to make it display whatever the user cares about.
         print "\e[22t\033]0;$_ $hostname\007";
         $self->{title_stack}++;
         $SIG{INT} = sub { for (1..$self->{title_stack}) { print "\e[23t"; } $self->{title_stack}=0; };
+
         $self->expect($self->{timeout},
                 @{$self->_banners},
                 @dynamic,
-                # fucking shitty allied telesyn
-                [ qr/User Access Verification - RADIUS/ => sub { $allied_shit=1;
-                    $self->send("$rtr->{user}\r".$self->password()."\r"); exp_continue } ],
-                [ qr/User Access Verification - Local/ => sub {
-                    $self->send("$rtr->{user}\r".$self->password()."\r"); $self->connected(1); last METHOD; } ],
                 [ $rtr->{username_prompt} => sub {
-                    $self->send("$rtr->{user}\r") unless($allied_shit); exp_continue; } ],
+                    $self->send("$rtr->{user}\r");
+                    exp_continue;
+                } ],
                 [ $rtr->{password_prompt} => sub {
-                    if ($allied_shit) {
-                        exp_continue;
-                    } else {
-                        $self->send($self->password() ."\r");
-                        $self->connected(1);
-                        last METHOD;
-                    }
+                    $self->send($self->password() ."\r");
+                    $self->connected(1);
+                    last METHOD;
                 } ],
                 [ qr/Name or service not known|hostname nor servname provided, or not known|could not resolve / => sub
                     {
@@ -653,13 +674,15 @@ sub login {
                        }
                     }
                 ],
-                [ qr/Corrupted/ => sub { next METHOD; } ],
                 # almost never needed anymore.  Some people might not want a
                 # fallback to des.  If anyone does we need to make it optional
                 #[ qr/cipher type \S+ not supported/ => sub { $rtr->{ciphertype}="des"; redo METHOD; } ],
-                [ qr/ssh_exchange_identification/ => sub { next METHOD; } ],
-                [ qr/[Cc]onnection (refused|closed)/ => sub { next METHOD; } ],
-                [ qr/key_verify failed/ => sub { next METHOD; } ],
+
+                # removing these 4, they should be handled by eof anyway
+                #[ qr/ssh_exchange_identification/ => sub { next METHOD; } ],
+                #[ qr/[Cc]onnection (refused|closed)/ => sub { next METHOD; } ],
+                #[ qr/key_verify failed/ => sub { next METHOD; } ],
+                #[ qr/Corrupted/ => sub { next METHOD; } ],
                 [ 'eof' => sub { next METHOD; } ],
                 [ 'timeout' => sub { next METHOD; } ],
         );
@@ -669,7 +692,7 @@ sub login {
     $rtr->{prompt} ||= '#';
 
     warn "Connection to $hostname failed.\n" if !$self->connected;
-    return $self->connected;
+    return $self;
 }
 
 =head2 logging
@@ -683,277 +706,13 @@ Turns on logging for this session.  If you specify a filename it will log to
 
 sub logging {
     my $self = shift;
+    return $self if (!$self->{opts}->{l});
+
     my $file = shift;
     $file ||= $self->{hostname};
     unlink ("/tmp/$file.log") if (-f "/tmp/$file.log");
     $self->session->log_file("/tmp/$file.log");
-}
-
-=head2 interact
-
-    $self->interact($input, $escape);
-
-This is a copy of Expect's interact() command.  It's been rewritten in parts
-to customize it for our needs, but might be very similar otherwise.  It's
-mainly a setup script for the call to interconnect().
-
-=cut
-
-sub interact {
-    my $self = shift;
-    my $session = $self->{'session'};
-    my $in_object = shift;
-    my $escape_sequence = shift;
-
-    my @old_group = $session->set_group();
-    # we know the input is STDIN and that it's an object.
-    my $out_object = Expect->exp_init(\*STDOUT);
-    $out_object->manual_stty(1);
-    $session->set_group($out_object);
-
-    $in_object->set_group($session);
-    $in_object->set_seq($escape_sequence,undef) if defined($escape_sequence);
-    # interconnect normally sets stty -echo raw. Interact really sort
-    # of implies we don't do that by default. If anyone wanted to they could
-    # set it before calling interact, of use interconnect directly.
-    my $old_manual_stty_val = $session->manual_stty();
-    $session->manual_stty(1);
-    # I think this is right. Don't send stuff from in_obj to stdout by default.
-    # in theory whatever 'session' is should echo what's going on.
-    my $old_log_stdout_val = $session->log_stdout();
-    $session->log_stdout(0);
-    $in_object->log_stdout(0);
-    # Allow for the setting of an optional EOF escape function.
-    #  $in_object->set_seq('EOF',undef);
-    #  $session->set_seq('EOF',undef);
-    $self->interconnect($in_object);
-    $session->log_stdout($old_log_stdout_val);
-    $session->set_group(@old_group);
-    # If old_group was undef, make sure that occurs. This is a slight hack since
-    # it modifies the value directly.
-    # Normally an undef passed to set_group will return the current groups.
-    # It is possible that it may be of worth to make it possible to undef
-    # The current group without doing this.
-    unless (@old_group) {
-        @{${*$session}{exp_Listen_Group}} = ();
-    }
-    $session->manual_stty($old_manual_stty_val);
-}
-
-=head2 interconnect
-
-    $self->interconnect(@handles);
-
-This is a copy of Expect's interconnect() method that has been modified to
-support the new things we needed.  The main differences are colorize and our
-winch handler.
-
-Future versions might support AnyEvent::IO or AnyEvent::Socket, but I might
-contribute that back to Expect's core stuff.  I might also try to figure out
-how to make the colorize stuff more hookable so we could use Expect's methods
-without rewriting them.
-
-=cut
-
-sub interconnect {
-    my $self = shift;
-    my @handles = ($self->{'session'}, $_[0]);
-
-    my ( $nread );
-    my ( $rout, $emask, $eout );
-    my ( $escape_character_buffer );
-    my ( $read_mask, $temp_mask ) = ( '', '' );
-
-    # Get read/write handles
-    foreach my $handle (@handles) {
-        $temp_mask = '';
-        vec( $temp_mask, $handle->fileno(), 1 ) = 1;
-        $read_mask = $read_mask | $temp_mask;
-    }
-    if ($Expect::Debug) {
-        print STDERR "Read handles:\r\n";
-        foreach my $handle (@handles) {
-            print STDERR "\tRead handle: ";
-            print STDERR "'${*$handle}{exp_Pty_Handle}'\r\n";
-            print STDERR "\t\tListen Handles:";
-            foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-                print STDERR " '${*$write_handle}{exp_Pty_Handle}'";
-            }
-            print STDERR ".\r\n";
-        }
-    }
-
-    #  I think if we don't set raw/-echo here we may have trouble. We don't
-    # want a bunch of echoing crap making all the handles jabber at each other.
-    foreach my $handle (@handles) {
-        unless ( ${*$handle}{"exp_Manual_Stty"} ) {
-
-            # This is probably O/S specific.
-            ${*$handle}{exp_Stored_Stty} = $handle->exp_stty('-g');
-            print STDERR "Setting tty for ${*$handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
-                if ${*$handle}{"exp_Debug"};
-            $handle->exp_stty("raw -echo");
-        }
-        foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-            unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
-                ${*$write_handle}{exp_Stored_Stty} =
-                    $write_handle->exp_stty('-g');
-                print STDERR "Setting ${*$write_handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
-                    if ${*$handle}{"exp_Debug"};
-                $write_handle->exp_stty("raw -echo");
-            }
-        }
-    }
-
-    print STDERR "Attempting interconnection\r\n" if $Expect::Debug;
-
-    # Wait until the process dies or we get EOF
-    # In the case of !${*$handle}{exp_Pid} it means
-    # the handle was exp_inited instead of spawned.
-    CONNECT_LOOP:
-    while (1) {
-
-        # test each handle to see if it's still alive.
-        foreach my $read_handle (@handles) {
-            waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
-                if ( exists( ${*$read_handle}{exp_Pid} )
-                and ${*$read_handle}{exp_Pid} );
-            if (    exists( ${*$read_handle}{exp_Pid} )
-                and ( ${*$read_handle}{exp_Pid} )
-                and ( !kill( 0, ${*$read_handle}{exp_Pid} ) ) )
-            {
-                print STDERR
-                    "Got EOF (${*$read_handle}{exp_Pty_Handle} died) reading ${*$read_handle}{exp_Pty_Handle}\r\n"
-                    if ${*$read_handle}{"exp_Debug"};
-                last CONNECT_LOOP
-                    unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
-                last CONNECT_LOOP
-                    unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
-                    ( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
-            }
-        }
-
-        my $nfound = select( $rout = $read_mask, undef, $eout = $emask, undef );
-
-        # Is there anything to share?  May be -1 if interrupted by a signal...
-        $self->_winch() if $_winch_it;
-        next CONNECT_LOOP if not defined $nfound or $nfound < 1;
-
-        # Which handles have stuff?
-        my @bits = split( //, unpack( 'b*', $rout ) );
-        #$eout = 0 unless defined($eout);
-        #my @ebits = split( //, unpack( 'b*', $eout ) );
-        #    print "Ebits: $eout\r\n";
-        foreach my $read_handle (@handles) {
-            if ( $bits[ $read_handle->fileno() ] ) {
-                # it would be nice if we could say read until TELNET_GA or the
-                # equivilant, but that's not something we can be sure would be
-                # there.  Cisco doesn't always fill the buffers just because
-                # they could, so there is a chance even though we tell them we
-                # can accept 10k they'll only send 1k and we split the middle
-                # of a regex.  With escape sequences that isn't a big deal.
-                # With colorizing it causes problems because it's data already
-                # written to the screen so you can't take it back (without big
-                # work)
-                $nread = sysread(
-                    $read_handle, ${*$read_handle}{exp_Pty_Buffer},
-                    10240
-                );
-
-                # don't bother trying to colorize input from the user
-                if ($read_handle->fileno() != $self->{stdin_fileno}) {
-                    foreach my $color (@{$self->{colors}}) {
-                        ${*$read_handle}{exp_Pty_Buffer} = $color->colorize(${*$read_handle}{exp_Pty_Buffer});
-                    }
-                }
-                # Appease perl -w
-                $nread = 0 unless defined($nread);
-                print STDERR "interconnect: read $nread byte(s) from ${*$read_handle}{exp_Pty_Handle}.\r\n"
-                    if ${*$read_handle}{"exp_Debug"} > 1;
-
-                # Test for escape seq. before printing.
-                # Appease perl -w
-                $escape_character_buffer = ''
-                    unless defined($escape_character_buffer);
-                $escape_character_buffer .= ${*$read_handle}{exp_Pty_Buffer};
-                foreach my $escape_sequence ( keys( %{ ${*$read_handle}{exp_Function} } ) ) {
-                    print STDERR "Tested escape sequence $escape_sequence from ${*$read_handle}{exp_Pty_Handle}"
-                        if ${*$read_handle}{"exp_Debug"} > 1;
-
-                    # Make sure it doesn't grow out of bounds.
-                    $escape_character_buffer = $read_handle->_trim_length(
-                        $escape_character_buffer,
-                        ${*$read_handle}{"exp_Max_Accum"}
-                    ) if ( ${*$read_handle}{"exp_Max_Accum"} );
-                    if ( $escape_character_buffer =~ /($escape_sequence)/ ) {
-                        my $match = $1;
-                        if ( ${*$read_handle}{"exp_Debug"} ) {
-                            print STDERR
-                                "\r\ninterconnect got escape sequence from ${*$read_handle}{exp_Pty_Handle}.\r\n";
-
-                            # I'm going to make the esc. seq. pretty because it will
-                            # probably contain unprintable characters.
-                            print STDERR "\tEscape Sequence: '"
-                                . _trim_length(
-                                undef,
-                                _make_readable($escape_sequence)
-                                ) . "'\r\n";
-                            print STDERR "\tMatched by string: '" . _trim_length( undef, _make_readable($match) ) . "'\r\n";
-                        }
-
-                        # Print out stuff before the escape.
-                        # Keep in mind that the sequence may have been split up
-                        # over several reads.
-                        # Let's get rid of it from this read. If part of it was
-                        # in the last read there's not a lot we can do about it now.
-                        if ( ${*$read_handle}{exp_Pty_Buffer} =~ /([\w\W]*)($escape_sequence)/ ) {
-                            $read_handle->_print_handles($1);
-                        } else {
-                            $read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
-                        }
-
-                        # Clear the buffer so no more matches can be made and it will
-                        # only be printed one time.
-                        ${*$read_handle}{exp_Pty_Buffer} = '';
-                        $escape_character_buffer = '';
-
-                        # Do the function here. Must return non-zero to continue.
-                        # More cool syntax. Maybe I should turn these in to objects.
-                        last CONNECT_LOOP
-                            unless &{ ${ ${*$read_handle}{exp_Function} }{$escape_sequence} }
-                            ( @{ ${ ${*$read_handle}{exp_Parameters} }{$escape_sequence} } );
-                    }
-                }
-                $nread = 0 unless defined($nread); # Appease perl -w?
-                waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
-                    if ( defined( ${*$read_handle}{exp_Pid} )
-                    && ${*$read_handle}{exp_Pid} );
-                if ( $nread == 0 ) {
-                    print STDERR "Got EOF reading ${*$read_handle}{exp_Pty_Handle}\r\n"
-                        if ${*$read_handle}{"exp_Debug"};
-                    last CONNECT_LOOP
-                        unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
-                    last CONNECT_LOOP
-                        unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
-                        ( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
-                }
-                last CONNECT_LOOP if ( $nread < 0 ); # This would be an error
-                $read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
-            }
-        }
-    }
-    foreach my $handle (@handles) {
-        unless ( ${*$handle}{"exp_Manual_Stty"} ) {
-            $handle->exp_stty( ${*$handle}{exp_Stored_Stty} );
-        }
-        foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-            unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
-                $write_handle->exp_stty( ${*$write_handle}{exp_Stored_Stty} );
-            }
-        }
-    }
-
-    return;
+    return $self;
 }
 
 =head2 run_commands
@@ -973,9 +732,14 @@ sub run_commands {
         $arg =~ s/\\r/\r/g; # fix for reload\ry.  I believe 'perldoc quotemeta' explains why this happens
         chomp($arg);
         $self->send("$arg\r");
-        $self->expect($self->{timeout},'-re', $self->profile->{prompt});
+        $self->expect($self->{timeout},
+            [ $self->profile->{prompt} => sub { } ],
+            [ 'eof' => sub { die "EOF From host.\n"; } ],
+        );
         sleep($opts->{s}) if ($opts->{s});
     }
+
+    return $self;
 }
 
 =head2 control_loop
@@ -984,7 +748,7 @@ sub run_commands {
 
 This is where control should be passed once the session is logged in.  This
 handles CLI commands passed via the -c option, or scripts executed with the -x
-option.  It also handles autocommands passed via either option -a on the
+option.  It also handles auto commands passed via either option -a on the
 command line, or via autocmds in the profile.
 
 Calling this without any commands will just run interact()
@@ -1025,7 +789,7 @@ sub control_loop {
         if (ref($pagercmd) eq 'CODE') {
             $pagercmd->();
         } elsif ($pagercmd) {
-            $self->run_commands("$pagercmd\r");
+            $self->run_commands("$pagercmd");
         }
         $self->run_commands(@args);
         $self->send($profile->{logoutcmd} ."\r");
@@ -1033,9 +797,24 @@ sub control_loop {
         die 'STDIN Not a tty' if (!POSIX::isatty($self->{stdin}));
         if ($autocmds) {
             $self->expect($self->{timeout},'-re',$prompt);
-            $self->run_commands(@$autocmds);
+            eval { $self->run_commands(@$autocmds); };
+            return $self if ($@);
         }
-        $self->interact($self->{stdin}, '\cD');
+
+        my $color_cb = sub {
+            my ($session) = @_;
+            $self->_winch() if $_winch_it;
+            ${*$session}{exp_Pty_Buffer} = $self->{colors}->colorize(${*$session}{exp_Pty_Buffer});
+            return 1;
+        };
+
+        my $sleep_cb = sub {
+            sleep($self->{opts}->{S});
+            return 1;
+        };
+        $self->session->set_cb($self->session,$color_cb, [ \${$self->session} ]);
+        $self->{stdin}->set_seq("\r",$sleep_cb) if ($self->{opts}->{S});
+        $self->session->interact($self->{stdin}, '\cD');
         # q\b is to end anything that's at a More prompt or other dialog and
         # get you back to the command prompt
         # would be nice to detect if the session is closing down and not send
@@ -1044,6 +823,7 @@ sub control_loop {
         # we're kinda stuck doing this.
         $self->send("q\b" . $profile->{logoutcmd}. "\r");
     }
+    return $self;
 }
 
 1;
