@@ -6,7 +6,7 @@ use warnings;
 
 =head2 new
 
-    my $color = App::Tel::CiscoPingRainbowColors->new;
+    my $color = App::Tel::Color::CiscoPingRainbow->new;
 
 New color object..
 
@@ -16,15 +16,18 @@ sub new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
 
-    return bless( {
+    my $self = bless( {
         'current_color' => 0,
         'ping' => 0,
     }, $class);
+
+    $self->{'color_count'} = scalar $self->get_colors();
+    return $self;
 }
 
 sub _next_color {
     my $self = shift;
-    return colored(shift, $App::Tel::ColorObject::colors[$self->{current_color}++ % scalar(@App::Tel::ColorObject::colors)]);
+    return colored(@_, $self->get_colors($self->{current_color}++ % $self->{color_count}));
 }
 
 =head2 colorize
@@ -37,25 +40,22 @@ Given a line of text from a cisco router, this will try to colorize it.
 
 
 sub colorize {
-    my $self = shift;
-    $_ = shift;
+    my ($self, $text) = @_;
 
     # We need to define a start point in the buffer that the dot regex isn't
     # allowed to search before so that this line and ones before it with
     # dots don't get red dots.
-    if (/Sending \d+, \d+-byte ICMP Echos to/) {
+    if ($text =~ /Sending \d+, \d+-byte ICMP Echos to/) {
         $self->{ping}=1;
-    }
-
-    if ($self->{ping}) {
-            s/(\!)/$self->_next_color($1)/eg;
-            s/(\.)/colored('.', 'red')/eg;
-        if (/Success rate is/) {
+    } elsif ($self->{ping}) {
+        $text =~ s/(\!)/$self->_next_color($1)/eg;
+        $text =~ s/(\.)/colored('.', 'red')/eg;
+        if ($text =~ /Success rate is/) {
             $self->{ping}=0;
             $self->{current_color}=0;
         }
     }
-    return $_;
+    return $text;
 }
 
 1;
