@@ -11,13 +11,17 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw();
 our @EXPORT_OK = qw ( keyring input_password );
 
+# unit test override variables
+our $appname = 'tel script';
+our $_mock = 0;
+
 =head1 NAME
 
 App::Tel::Passwd - Methods for managing App::Tel::Passwd:: modules
 
 =cut
 
-my $plugins = [ 'KeePass', 'PWSafe', 'Pass' ];
+my $plugins = [ 'KeePass', 'PWSafe', 'Pass', 'KeyRing', 'Mock' ];
 
 =head2 load_module
 
@@ -49,16 +53,26 @@ line and returns it.
 
 =cut
 
-
 sub input_password {
     my $prompt = shift;
     $prompt ||= '';
-    die 'STDIN not a tty' if (!POSIX::isatty(\*STDIN));
-    my $old_mode=IO::Stty::stty(\*STDIN,'-g');
+    my $old_mode;
+    # uncoverable branch true
+    if (!$_mock) {
+        die 'STDIN not a tty' if (!POSIX::isatty(\*STDIN));
+        $old_mode=IO::Stty::stty(\*STDIN,'-g');
+        IO::Stty::stty(\*STDIN,'-echo');
+    }
     print "Enter password for $prompt: ";
-    IO::Stty::stty(\*STDIN,'-echo');
-    chomp(my $password=<STDIN>);
-    IO::Stty::stty(\*STDIN,$old_mode);
+    my $password=<STDIN>;
+
+    chomp($password);
+    # uncoverable branch true
+    if (!$_mock) {
+        IO::Stty::stty(\*STDIN,$old_mode);
+    } else {
+        print "\n";
+    }
     return $password;
 }
 
@@ -75,10 +89,9 @@ try to store it in the keyring.
 sub keyring {
     my ($user, $domain, $group) = @_;
 
-    # TODO: needs to warn on some kind of failure?
-    eval {
+    my $p = eval {
         load Passwd::Keyring::Auto, 'get_keyring';
-        my $keyring = get_keyring(app=>"tel script", group=>$group);
+        my $keyring = get_keyring(app=>$appname, group=>$group);
         my $pass = $keyring->get_password($user, $domain);
         if (!$pass) {
             $pass = input_password($domain);
@@ -86,6 +99,9 @@ sub keyring {
         }
         return $pass;
     };
+
+    warn $@ if ($@);
+    return $p;
 }
 
 =head2 load_from_profile
